@@ -1,4 +1,5 @@
 import os
+import platform
 from typing import List
 
 import aria2p
@@ -8,6 +9,7 @@ from aria2p import Options
 from aliyunpan.api.core import AliyunPan
 from aliyunpan.api.models import *
 from aliyunpan.api.req import *
+from aliyunpan.api.type import Share
 from aliyunpan.api.utils import *
 from aliyunpan.cli.config import Config
 from aliyunpan.cli.tui import AliyunpanTUI
@@ -22,7 +24,7 @@ class Commander:
     def __init__(self, init=True, *args, **kwargs):
         self._disk = AliyunPan()
         self._path_list = PathList(self._disk)
-        self._req = Req()
+        self._req = Req(self._disk)
         self._config = Config()
         self._task_config = Config(ROOT_DIR / Path('tasks.yaml'))
         self._share_link = 'aliyunpan://'
@@ -39,14 +41,18 @@ class Commander:
     def __del__(self):
         self._task_config.write(GLOBAL_VAR.tasks)
         if self._disk.refresh_token:
-            self._config.update('refresh_token', self._disk.refresh_token)
+            try:
+                self._config.update('refresh_token', self._disk.refresh_token)
+            except AttributeError:
+                pass
 
     def init(self, config_file=None, refresh_token=None, username=None, password=None, depth=3, timeout=None,
-             drive_id=None, album=False):
+             drive_id=None, album=False, share_id='', share_pwd=''):
         self._path_list.depth = depth
         self._req.timeout = timeout
         self._disk.drive_id = drive_id
         self._disk.album = album
+        self._disk._share = Share(share_id, share_pwd)
         config_file_list = list(
             filter(lambda x: get_real_path(x).is_file(), map(lambda x: get_real_path(x), self._config_set)))
         if config_file:
@@ -102,14 +108,18 @@ class Commander:
             file_info_list = self._path_list.get_file_info(self._disk.search(query))
         else:
             file_info_list = self._path_list.get_path_list(path, update=False)
-        for i in file_info_list:
+        for i, j in enumerate(file_info_list):
             if l:
-                if i.type:
-                    print(str_of_size(i.size), time.strftime('%d %b %H:%M', i.ctime), i.id, i.name)
+                if j.type:
+                    print(str_of_size(j.size), time.strftime('%d %b %H:%M', j.ctime), j.id, j.name, end='')
                 else:
-                    print('-', time.strftime('%d %b %H:%M', i.ctime), i.id, i.name)
+                    print('-', time.strftime('%d %b %H:%M', j.ctime), j.id, j.name, end='')
+                if i + 1 < len(file_info_list):
+                    print()
             else:
-                print(i.name, end='\t')
+                print(j.name, end='\t')
+        if platform.system() != 'Windows':
+            print()
 
     def get_path_list(self, path='root') -> List[FileInfo]:
         return self._path_list.get_path_list(path, update=False)
